!
!=======================================================================
!
! This file contains subroutines which prompt the user neatly to enter
! needed information via the terminal keyboard.
!
!            OFFICIAL DAO VERSION:  1991 April 18
!
!***********************************************************************
!
! Current contents
!
!   INQUIR  right justifies a prompting string to an arbitrary column
!   GETNAM  asks (neatly) for a filename, offering a default.
!   GETFIL  asks for the name and opens the file
!   GETDAT  asks (neatly) for real-number input.
!   GETYN   asks (neatly) for an answer, and accepts only 'Y', 'y',
!           'N', 'n', 'E', or 'e'.
!   GETCHR  asks (neatly) for a Character string
!
! If CTRL-Z is entered as input to any of the above, the routine will
! return directly to the calling routine, with a unique response.
!
!   TBLANK  types a blank line on the terminal.
!   SWITCH  removes the filename extension from an input filename and
!           attaches another character string in its place.
!   EXTEND  checks to see whether an input filename contains a filename
!           extension and, if not, attaches a default filename
!           extension.
!
!***********************************************************************
!
      subroutine  inquir (prompt, n)
!
!=======================================================================
!
! Simply type a prompting character string on the terminal,
! right-justified to column N and followed by a single blank, and
! suppress the carriage return.
!
! Argument
!
! PROMPT  (INPUT) is the character string.
!      N  (INPUT) is the column number to contain the last character.
!
!=======================================================================
!
      implicit none
      character(len=*) :: prompt
      character(len=10) :: format
      integer :: n

      write(format,"('A',I2.2)") n
      write(*,format,advance="no" ) prompt

      END!
!
!#######################################################################
!
      subroutine  getnam (prompt, file)
!
!=======================================================================
!
! Prompt the user neatly to type in a filename.
!
! Arguments
!
! PROMPT  (INPUT) the character string to be typed on the terminal
!         (right-justified to column 50).
!
!   FILE  (INPUT/OUTPUT) the filename.  On input, it is the default
!         filename, if any, to be offered to the user.  On output,
!         it is what the user wants.
!
!=======================================================================
!
      implicit none
      character(len=*)  :: prompt, file

      integer :: ierr

do
  call inquir( prompt, 50)
  read(*,"(A)",end=9100,iostat=ierr) file
  if( ierr /= 0 ) then
     call stupid ('Error reading filename.')
  else
     return
  endif
enddo  
      
 9100 file='END OF FILE'
      return

      end!
!
!#######################################################################
!
      subroutine  getfil (prompt, file, lun, status)
      implicit none
      character(len=*) :: prompt, file
      character :: status
      integer lun, istat

 1000 call getnam (prompt, file)
      if ((status .eq. 'N') .or. (status .eq. 'n')) then
         call outfil (lun, file, istat)
      else
         call infile (lun, file, istat)
      end if
      if (istat /= 0) go to 1000
      return
      end!
!
!#######################################################################
!
      subroutine  getdat (prompt, data, n)
!
!=======================================================================
!
! Prompt the user to input numerical data.
!
! Arguments
!
! PROMPT  (INPUT) is the prompting character string to be typed out,
!         right-justified to column 50.
!
!   DATA  (OUTPUT) is a vector of dimension N, to return the numerical
!         values entered by the user to the calling program.
!
!      N  (INPUT) is the number of individual data items the user
!         is required to enter.
!
! Accept only numerical data, or CTRL-Z.  If CTRL-Z is entered, DATA(1)
! is returned to the calling program equal to -1.1E38.
!
!=======================================================================
!
      implicit none
      character(len=*) ::  prompt
      character(len=80) :: line
      integer :: n
      real :: data(n)
      integer :: ierr
!
!-----------------------------------------------------------------------
!
 1000 call inquir (prompt, 50)
      read (*,501,iostat=ierr,end=9100) line
  501 format (a)
      if (ierr /= 0) then
         call stupid ('Error reading data.')
         go to 1000
      end if
      if ((line(1:1) == 'E') .or. (line(1:1) == 'e')) go to 9100
      read (line,*,iostat=ierr) data
      if (ierr /= 0) then
         call stupid ('Error reading data.')
         go to 1000
      end if
      return
!
!  CTRL-Z was entered
!
 9100 data(1)=-1.1e38
      return

      end!
!
!#######################################################################
!
      subroutine  getyn (prompt, answer)
!
!=======================================================================
!
! A subroutine to get the user's response to a yes/no question.
!
! Arguments
!
! PROMPT  (INPUT) is the prompting character string, to be typed out
!         right-justified to column 50.
!
! ANSWER  (OUTPUT) is the user's response to the question.
!
! Accept only 'Y', 'y', 'N', 'n', or CTRL-Z as a valid response:
! If 'Y' or 'y', return ANSWER = 'Y' to the calling routine.
! If 'N' or 'n', return ANSWER = 'N' to the calling routine.
! If CTRL-Z, return ANSWER = 'E' to the calling routine.
!
!=======================================================================
!
      implicit none
      character(len=*) :: prompt 
      character :: ANSWER
      integer :: ierr

 1000 call inquir (prompt, 50)
      read (*,510,iostat=ierr,end=9100) answer
  510 format (a1)
      if (ierr /= 0) then
         call stupid ('Error reading answer.')
         go to 1000
      end if
      if (answer == 'y') answer='Y'
      if (answer == 'n') answer='N'
      if (answer == 'e') answer='E'
      if ((answer /= 'Y') .and. (answer /= 'N') .and. (answer /= 'E')) then
         call stupid ('Error understanding answer.')
         go to 1000
      end if
      return                                             ! Normal return
!
! CTRL-Z was entered.
!
 9100 answer='E'
      return

      end!
!
!#######################################################################
!
      subroutine  getchr (prompt, answer, n)
!
!=======================================================================
!
! A subroutine to get a character string from the user.
!
! Arguments
!
! PROMPT  (INPUT) is the prompting character string, to be typed out
!         right-justified to column 50.
!
! ANSWER  (OUTPUT) is the user's response.
!
!      N  (OUTPUT) is the number of characters in the user's response.
!
! Accept about anything.
!
!=======================================================================
!
      implicit none
      character(len=*) :: prompt 
      character :: answer
      integer :: i, j, n, ierr

 1000 call inquir (prompt, 50)
      read (*,510,iostat=ierr,end=9000) answer
  510 format (a)
      if (ierr /= 0) then
         call stupid ('Error reading input.')
         go to 1000
      end if
      n = 0
      do i = 1,len(answer)
         j = ichar(answer(i:i))
         if ((j /= 0) .and. (j /= 32)) n = i
      end do
      return
 9000 n = -1
      return

      end!
!
!#######################################################################
!
      subroutine  tblank
!
!=======================================================================
!
! A subroutine to type a blank line on the terminal.
!
!=======================================================================
!
      implicit none
      write (*,*)
      return

      end!
!
!#######################################################################
!
!      character*30  function  switch  (file, addend)
!  !!!!  The declaration 'character*30' needs assumption of the constant 
!    string length, this assumption remove using of a subroutine:
!    
     subroutine switch (file, addend) 
!                                       file(input/output)
!
!=======================================================================
!
! Subroutine to chop off a filename extension from an input filename
! character string, and append a new character string to the end.
!
! Arguments
!
!   FILE (INPUT) is a character string containing a filename.
!
! ADDEND (INPUT) is the character string to be pasted onto the end
!        of the character string obtained by removing a directory
!        name and the period and all that follows from the input
!        filename.
!
! SWITCH is the filename part of NAFILE, with any directory name
!        contained in brackets chopped off from the front, and with
!        the character string ADDEND substituted for the period and
!        filename extension of the input string.
!
!!!! Changed by FH, remove and append a new suffix on the end only 
!=======================================================================
!
      implicit none
      character(len=*) :: file, addend
      integer :: i

      do i = len(file),1,-1
         if( file(i:i) == '.' )then
            file = file(:i-1)//addend
            return                              ! regular return
         endif
      enddo

! error
      write (*,691) char(7), file, addend
  691 format (/' Error creating file name: ', A1, 5X, A30, 5X, A10/)
      call byebye
      end!
!
!#######################################################################
!
      character*30  function  extend (file,ext)
!
!=======================================================================
!
! A function to see whether a filename contained in an input character
! string contains a filename extension.  If it does, leave it alone.
! If not, then attach the filename extension contained in EXT.
!
! Arguments:
!
! FILE a character string presumably containing a filename.
!
!  EXT a character string containing a default filename extension.
!
!=======================================================================
!
      implicit none
      integer :: len

      character(len=*) :: file, ext
      integer :: i, j, l
      l = len(file)
      i=0
 1000 i=i+1
      if (i > l) go to 9100
      if ((file(i:i) == '[') .or. (file(i:i) == '<')) then
!
! If a left bracket is found then a directory name is present.  Find the
! corresponding right bracket, ignoring any periods met along the way.
!
 1001    i=i+1
         if ((file(i:i) /= ']') .and. (file(i:i) /= '>')) go to 1001
         go to 1000
      else
!
! If a period is found, then a filename extension is present.
!
         if (file(i:i) == '.') then
            extend=file
            return
         end if
!
! If a non-blank, non-null character is found, then the filename
! continues to at least this point in the string.
!
         if ((file(i:i) /= ' ') .and. (file(i:i) /= char(0))) j=i
         if (i < l) go to 1000
      end if
!
! No period was found, so append "." and EXT to the end of the
! character string.
!
      extend=file(1:j)//'.'//ext
      return
 9100 write (*,691) char(7), file, ext
  691 format (/' Error creating file name: ', A1, 5X, A30, 5X, A3/)
      call byebye
      end!
!
!#######################################################################
!
      subroutine  rdstar (lu, nl, id, x, y, amag, sky)
!
!=======================================================================
!
! Read in an ID number, x and y coordinates, magnitude, and sky value
! for the next star in the input file, whatever the file type (i.e.,
! NL = 1, 2, or 3).  If an end of file is encountered, set the star ID
! to a negative number and return; if a blank line is encountered, the
! star ID will be zero.  LU is the logical unit number to be used; the
! other arguments are obvious.
!
!=======================================================================
!
      implicit none
      character(len=133) :: line
      real :: x, y, amag, sky, dummy
      integer :: lu, n, istat, id, nl

  900 call rdchar (lu, line, n, istat)
      if (istat > 0) then
         id = -1   !  ID = -1 means END OF FILE
         return
      else if (istat < 0) then
         call stupid ('Unable to read line from input file')
         call oops
      end if

      if (n <= 1) then
         id = 0                 ! Blank line
         return
      end if

      n = n+1
      line(n:n) = '/'
      if (nl == 1) then
         read (line(2:n),*,err=1000) id, x, y, amag, dummy, sky
      else if (nl .eq. 2) then
         read (line(2:n),*,err=1000,end=2000) id, x, y, amag
         if (id /= 0) read(lu,*,err=1000) sky
      else if (nl == 3) then
         read (line(2:n),*,err=1000) id, x, y, amag, sky
      end if
      return                                             ! Normal return
 1000 call stupid ('WARNING:  Corrupt star data encountered in input.')
      go to 900
!
!-----------------------------------------------------------------------
!
! End of file was encountered.
!
 2000 id=-1
      return

      end!
!
!#######################################################################
!
      subroutine  rdhead (lun, nl, ncol, nrow, lobad, hibad, &
           thresh, ap1, phpadu, readns, frad)
!
!=======================================================================
!
! Read file header from a sequential data file on the disk.  If the
! file has no header, rewind to beginning.
!
! Input argument
!
!    LUN  is the logical unit number of the disk file.
!
!     NL  if NL is greater than zero on input, then in the event that
!         the header is missing from the input file, the missing
!         header information will be requested via the keyboard.
!         If NL is equal to zero on input, only NCOL and NROW
!         will be requrested.  If NL is a negative number on input,
!         missing information will NOT be requested of the user.
!
! Output arguments
!
!     NL  is a code indicating the file type:
!            NL = 3 a group file
!            NL = 2 an aperture photometry file
!            NL = 1 other (output from FIND, PEAK, or NSTAR)
!            NL = 0 a file without a header
!
!    NCOL  is the number of columns in the picture (length in x)
!
!    NROW  is the number of rows in the picture (length in y)
!
!  THRESH  is the star-finding threshold that was used (from FIND)
!
!     AP1  is the radius of the first aperture (from PHOT)
!
!  PHPADU  is the number of photons per ADU that was given(from PHOT)
!
!  READNS  is the readout noise per pixel in ADU (from PHOT)
!
!   LOBAD  is the low bad pixel limit that was used (from FIND)
!
!   HIBAD  is the high bad pixel limit that was used (from FIND)
!
!    FRAD  is the fitting radius (from OPTION, used by PSF, PEAK,
!          and NSTAR)
!
!=======================================================================
!
      implicit none
      character(len=133) :: line
      character(len=4) :: first4, test
      real :: lobad, hibad, thresh, ap1, phpadu, readns, frad, x
      integer :: lun, nl, ncol, nrow, n, istat, nlin

      read (lun,900) first4, test
  900 format (a4, 9x, a4)
      if (first4 /= ' NL ') go to 8000               ! No header lines
      if (test == '  LO') then
!
! Latest DAOPHOT II file header.
!
         call rdchar (lun, line, n, istat)
         n = n + 1
         line(n:n) = '/'                    ! <-- potentialy dangerous !!!!!
         read (line,*) nl, ncol, nrow, lobad, hibad, thresh, &
                       ap1, phpadu, readns, frad
      else if ((test /= 'LOWB') .and. (test /= 'OWBA')) then
!
! Obsolete file header.
!
         read (lun,901) nlin, ncol, nrow, thresh, ap1, phpadu, readns, &
              lobad, frad
  901    format (1x, i2, 2i4, 7f8.1)
         if (nl >= 0) then
            if (ap1 <= 0.0) then
 1010          call getdat ('Lowest good data-value:', lobad, 1)
               if (lobad <= -1.0e38) go to 1010
            end if
 1020       call getdat ('Highest good data-value:', hibad, 1)
            if (hibad <= 0.0) go to 1020
         end if
         nl=nlin
      else
         read (lun,901) nl, ncol, nrow, lobad, hibad, thresh, ap1, &
                        phpadu, readns, frad
      end if
      read (lun,900)
      return                                           ! normal return

 8000 rewind (lun)
      if (nl >= 0) then
 8001    call getdat ('Number of columns in the image:', x, 1)
         if (x < 0.5) go to 8001
         ncol = nint(x)
 8002    call getdat ('Number of rows in the image:', x, 1)
         if (x < 0.5) go to 8002
         nrow = nint(x)
      end if

      if (nl > 0) then
 8003    call getdat ('Readout noise (ADU):', readns, 1)
         if (readns <= 0.) go to 8003
 8006    call getdat ('Gain (photons per ADU):', phpadu, 1)
         if (phpadu <= 0.) go to 8006
 8010    call getdat ('Lowest good data-value:', lobad, 1)
         if (lobad <= -1.e38) go to 8010
 8020    call getdat ('Highest good data-value:', hibad, 1)
         if (hibad <= -1.e38) go to 8020
      end if
      nl=0
      return

      end!
!
!#######################################################################
!
      subroutine  wrhead (lun, nl, ncol, nrow, items, lobad, hibad, &
           thresh, ap1, phpadu, readns, frad)
!
!=======================================================================
!
! Subroutine to write a standard header into an output sequential
! data file on the disk.  Same as RDHEAD, except that all the output
! arguments are now input arguments.  ITEMS tells how many of the
! individual arguments are to be written into the header.
!
!              Latest DAOPHOT II version:  1991 January 15
!=======================================================================
!
      implicit none
      character(len=8) :: head(7)
      real :: A(7)

      real :: lobad, hibad, thresh, ap1, phpadu, readns, frad
      integer :: lun, nl, ncol, nrow, items, i
      data  head /'  LOWBAD', ' HIGHBAD', '  THRESH', '     AP1', &
           '  PH/ADU', '  RNOISE', '    FRAD'/
!
!-----------------------------------------------------------------------
!
      a(1)=lobad
      a(2)=hibad
      a(3)=thresh
      a(4)=ap1
      a(5)=phpadu
      a(6)=readns
      a(7)=frad
      write (lun,900) (head(i), i=1,items)
  900 format (' NL   NX   NY', 8A8)
      write (lun,901) nl, ncol, nrow, (a(i), i=1,items)
  901 format (1x, i2, 2i5, 2f8.1, 5f8.2)
      write (lun,901)                             ! Write a blank line
      return                                        ! Normal return

      end!
!
!#######################################################################
!
      subroutine rdchar (lun, line, n, istat)
      implicit none
      character(len=*) :: line
      integer :: lun, n, istat

      read (lun,"(a)",end=8000,err=9000) line
      n = len_trim(line) 
      istat = 0
      return
 8000 line = ' '
      n = 0
      istat = 1  ! ISTAT = 1 means END OF FILE
      return
 9000 line = ' '
      n = 0
      istat = -1  ! ISTAT = -1 means ERROR
      return
      end!
!
!#######################################################################
!
      function extrct (string, iflag)
      implicit none
      integer :: len
      character(len=*) :: string
      real :: extrct
      integer :: l, iflag

      l = len(string)
      read (string, *, err=9000) extrct
      iflag = 0
      return
 9000 extrct = -1.1e38
      iflag = -1
      return
      end!
!
!#######################################################################
!
      subroutine  stupid (messag)
      implicit none
      character(len=*) :: messag
      character(len=1) :: bell
      bell = char(7)
      write (*,"(/1x, a, a1/)") messag, bell
      end!
!
!#######################################################################
!
      integer  function  icnvrt (astring)
!
!=======================================================================
!
! This little function is supposed to take two ASCII characters and
! express them as an integer in the range 0-(32**2-1) without
! distinguishing upper and lower case:
!
! AA = Aa = aA = aa = 0, AB = Ab = aB = ab = 1, BA = Ba = bA = ba = 32,
! etc.
!
! Argument
!
! ASTRING is a character string containing two ASCII characters.
!
!=======================================================================
!
      implicit none
      character(len=2) :: astring
!
!-----------------------------------------------------------------------
!
      icnvrt=32*mod(ichar(astring(1:1))-1,32) + mod(ichar(astring(2:2))-1,32)
      return

      end!
!
!#######################################################################
!
      subroutine  option (optfil, nopt, lbl, opt, omin, omax, prompt, istat)
!
!=======================================================================
!
! Acquire a set of optimization parameters, either from a disk file
! or from the keyboard.
!
!              OFFICIAL DAO VERSION:  1985 October 25
!
! Arguments
!
! OPTFIL (INPUT) a character string containing either (a) the name of
!         a disk file which will be offered as the default file from
!         which the parameter definitions are to be read (if OPTFIL =
!         ' ', no default file name will be offered; if OPTFIL =
!         'daophot.opt' and this file does not exist, no error message
!         is to be produced and the current contents of OPT are to be
!         left alone), or (b) the string 'KEYBOARD INPUT', in which
!         case the user is to be prompted to define parameter values
!         via the terminal.
!
!     OPT (INPUT/OUTPUT) numerical values for the NOPT user-definable
!         parameters.
!
!=======================================================================
!
      implicit none
      integer :: nopt
      integer, parameter  :: maxopt = 100
!
! Parameter:
!
! NOPT is the number of user-specifiable optional parameters.
!
      character(len=26) :: lbl(NOPT)
      real :: opt(NOPT), omin(NOPT), omax(NOPT)
      integer :: idopt(MAXOPT)

      real :: extrct
      integer :: icnvrt

      character(len=80) :: oline
      character(len=30) :: case
      character(len=*) :: optfil, prompt
      integer :: istat, npro, i, j, inunit, n, iopt
      logical :: change
!
!-----------------------------------------------------------------------
!
! Get ready to interpret the option.  For each of the defined labels,
! convert the first two non-blank characters to a number.
!
 1000 npro = len(prompt)
      do i=1,NOPT
         do j=1,26
            if (lbl(i)(j:j) /= ' ') exit
         end do
      enddo
      idopt(i)=icnvrt(lbl(i)(j:j+1))
!
! If OPTFIL equals anything but 'KEYBOARD INPUT', then treat it as a
! filename and try to open it and read the commands from it, then
! display the results; if it happens to be daophot.opt, return quietly
! afterwards; otherwise, after displaying what you've read, go back
! and prompt for new keyboard input.
!
      if (optfil == 'KEYBOARD INPUT') then
         inunit = 5
         call tblank
         write (*,"(1X, A26, ' =', F9.2, 4X, A26, ' =', F9.2)") (lbl(i), opt(i), i=1,NOPT)
         call tblank
         change = .false.
         go to 2000
      end if

      inunit = 2
      call infile (2, optfil, istat)
      if (istat /= 0) go to 3000
!
!-----------------------------------------------------------------------
!
! SECTION 2
!
! Loop to read and interpret commands one at a time.
!
 2000 if (inunit == 5) call inquir (prompt, npro)
      call rdchar (inunit, oline, n, istat)
      if (istat > 0) then
         if (inunit == 5) return
         go to 3000
      end if
      if (istat < 0) go to 2000
      if (n <= 3) go to 3000
!
! Now interpret this command line.
!
! First find the equals sign.  Set J to its numerical position in the
! string.
!
      do j = 3, n
        if (oline(j:j) == '=') go to 2020
      enddo
      call stupid ('The equals sign is missing.')
      go to 2000
!
! Now identify the parameter.
!
 2020 iopt=icnvrt(oline(1:2))
      do i = 1,nopt
        if (iopt == idopt(i)) go to 2040
      enddo
      call stupid ('Unrecognized parameter: '//oline(1:j-1))
      go to 2000
!
! Now decode the numeric constant and set the appropriate option to this
! value.  Then go back and obtain the next command line.
!
 2040 opt(i) = extrct (oline(j+1:n), istat)
      if (istat /= 0) then
           call stupid('Invalid numeric constant: '//OLINE(J+1:N))
         else
            change = .true.
      end if
      go to 2000
!
!-----------------------------------------------------------------------
!
! SECTION 3
!
! If file is neither 'daophot.opt' nor 'KEYBOARD INPUT', allow
! the user to enter modifications by hand.
!
 3000 if (inunit == 2) then
         call clfile (2)
         if (optfil /= case('daophot.opt')) then
            call tblank
            write (*,610) (lbl(i), opt(i), i=1,nopt)
  610    FORMAT (1X, A26, ' =', F9.2, 4X, A26, ' =', F9.2)

            call tblank
            inunit = 5
            change = .false.
            go to 2000
         end if
      end if
!
! Check that all parameters are acceptable.
!
      do i = 1,nopt
        if ((opt(i) < omin(i)) .or. (opt(i) > omax(i))) go to 4000
      enddo
!
! Type out all parameter values.
!
      if (change) then
         call tblank
         write (*,610) (lbl(i), opt(i), i=1,nopt)
      end if
      istat = 0
      return
!
!-----------------------------------------------------------------------
!
! SECTION 4
!
! There was some problem with one of the parameters.  Type it out and
! ask for the problem to be corrected.
!
 4000 call stupid ('Value unacceptable --- please re-enter')
      call inquir (lbl(i)//' =', 50)
      call rdchar (5, oline, n, istat)
      if (n <= 0) call oops
      if (istat > 0) go to 3000
      if (istat < 0) go to 4000
!
! Decode the constant.
!
      opt(i) = extrct(oline(1:n), istat)
      if (istat /= 0) go to 4000
      go to 3000
!
!-----------------------------------------------------------------------
!
! ERROR:  unable to open the disk file.
!
! If the file name is 'daophot.opt', no problem-- just check the
! default values for validity and return to the main program.
! Otherwise, type an error message and ask for a new file name.
!
! 9000 IF (OPTFIL .EQ. CASE('daophot.opt')) GO TO 3000
!      OLINE = 'Error opening input file '//OPTFIL
!      CALL STUPID (OLINE)
!      OPTFIL='KEYBOARD INPUT'
!      GO TO 1000
!
      end!
!
!#######################################################################
!
      subroutine check(lun,nl)
!
! Check whether an input file has a DAOPHOT data file header.
!
      implicit none
      character(len=4) :: first4
      integer :: nl, lun

      nl=1
      read(lun,200)first4
  200 format(a4)
      if(first4 /= ' NL ') go to 1000
      read(lun,201)nl
  201 format(1x,i2)
      read(lun,200)
      return
 1000 rewind(lun)
      return
      end!
